<?php
declare(strict_types=1);

namespace FiloBlu\Refilo\Model;

use Magento\Framework\App\ObjectManager;
use Magento\GiftCardAccount\Api\GiftCardAccountRepositoryInterface;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\GiftCardAccount\Model\Spi\GiftCardAccountManagerInterface;
use Magento\Framework\Serialize\Serializer\Json as JsonSerializer;

class GetGiftCardDataManagement implements \FiloBlu\Refilo\Api\GetGiftCardDataManagementInterface
{

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var GiftCardAccountManagerInterface
     */
    private $manager;

    /**
     * @var JsonSerializer
     */
    private $jsonSerializer;

    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        JsonSerializer $jsonSerializer,
        ?GiftCardAccountManagerInterface $manager = null
    ){
        $this->manager = $manager ?? ObjectManager::getInstance()->get(GiftCardAccountManagerInterface::class);
        $this->storeManager = $storeManager;
        $this->jsonSerializer = $jsonSerializer;
    }

    /**
     * {@inheritdoc}
     */
    public function getGetGiftCardData($giftCardCode)
    {
        //$current_website_id = $this->storeManager->getWebsite()->getId();
        $current_website_id = null;

        try {
            $giftCard = $this->manager->requestByCode(
                (string)$giftCardCode,
                $current_website_id,
                null,
                false,
                false
            );
            $website = $this->storeManager->getWebsite($giftCard->getWebsiteId());
            $gf_data = $giftCard->getData();
            $gf_data['currency_code'] = $website->getBaseCurrencyCode();

            // Fix types
            $gf_data['balance'] = (float)$gf_data['balance'];
            $gf_data['giftcardaccount_id'] = (int)$gf_data['giftcardaccount_id'];
            $gf_data['status'] = (int)$gf_data['status'];
            $gf_data['website_id'] = (int)$gf_data['website_id'];
            $gf_data['state'] = (int)$gf_data['state'];
            $gf_data['is_redeemable'] = (int)$gf_data['is_redeemable'];

            $json = $this->jsonSerializer->serialize($gf_data);
            return $json;

        } catch (\Exception $e) {
            throw new NoSuchEntityException(
                __("The gift card code is incorrect. Verify and try again."),
                $e
            );
        }

    }
}
